program SnakeTxt;

uses Crt;{delay,readkey}

const
     {screen parameters}
     ScrCX = 80;
     ScrCY = 50;

     {decimal number, decimal fraction size}
     BitShift = 5;

     {device constants}
     {transformations}
     TS_WORLD = 0;
     TS_VIEW = 1;
     TS_PROJECTION = 2;

     {math constants}
     KPi = 3.14159265;
     K2Pi = 2.0*KPi;

     {scene size}
     WorldCX = 10;
     WorldCY = 10;
type
    {VECTOR 3D STRUCT}
    Vector3D = Record
       x,y,z : real;    {x,y,z coordinates}
    end;

    {MATRIX 4x4 STRUCT}
    Matrix = Record
       M11,M12,M13,M14,
       M21,M22,M23,M24,
       M31,M32,M33,M34,
       M41,M42,M43,M44: real; { [4][4] items }
    end;

    {SNAKE STRUCT}
    PSnakeSegment = ^SnakeSegment;

    SnakeSegment = Record

      {segment position}
      vecPos : Vector3D;
      vecFrom: Vector3D;

      {segment info}
      length : real;
      turn   : real;{segment angle}
      iSin,iCos:real;{:=sincos(turn)}

      {next item in list}
      pNext  : PSnakeSegment;
    end;

    {RENDER DEVICE OBJECT}
    Device = Object

       {transformations state}
       mat,
       matWorld,matView,matProj : Matrix;

       {screen buffers}
       zbuffer : array[ 0 .. ScrCX*ScrCY ] of word;     {z-buffer}
       bbuffer : array[ 0 .. ScrCX*ScrCY ] of byte;     {backbuffer}
       tbuffer : array[ 0 .. ScrCX*ScrCY ] of char;     {text buffer}

       {current brush}
       color:byte;

       {inicialization}
       procedure InitDevice;
       procedure FreeDevice;

       {drawing}
       procedure Present;
       procedure Clear;

       procedure RenderLine( const v0,v1:Vector3D );
       procedure DrawText(const x,y:longint; const str:string);

       {states}
       procedure SetTransform( ts:integer; const m:Matrix );
       procedure SetBrushColor( b:byte );

       {transforming}
       procedure VectorProject( var vecOut:Vector3D; const vec:Vector3D );

    end;

    {HOUSE OBJECT}
    PHouse = ^House;
    House = Object
      {model}
      vMin,vMax : Vector3D;
      color     : byte;

      {next item}
      pNext     : PHouse;

      {scene}
      procedure Render;

      {collision}
      function  PtIsIn(const v:Vector3D):boolean;
    end;

    {GAME OBJECT}
    Game = Object

       {states}
       iTurn:real;
       iSin,iCos:real;

       {game state}
       iScore:word;
       length:real;
       bAlive:boolean;
       bExit :boolean;

       {snake}
       iSnake:PSnakeSegment;
       speed:real;

       {scene}
       houses : PHouse;

       apple  : Vector3D;

       {scene}
       procedure Draw;
       procedure FrameMove( flFrameLength:real );

       {actions}
       procedure NewGame;
       procedure FreeGame;
       procedure ResetApple;

       {events}
       procedure OnKey( iKey:char );

    end;
var
    {render device}
    iDevice:Device;

    {game object}
    iGame  :Game;

    {iS : longint;}

{ MATH }
{return min(a,b)}
function        Min( const a,b:longint ):longint;
begin
  if (a<b) then
  Min := a
  else
  Min := b;
end;
{return min(a,b)}
function        MinF( const a,b:real ):real;
begin
  if (a<b) then
  MinF := a
  else
  MinF := b;
end;
{return max(a,b)}
function        MaxF( const a,b:real ):real;
begin
  if (a>b) then
  MaxF := a
  else
  MaxF := b;
end;

{ VECTOR }
procedure       VectorAdd( var vecOut:Vector3D;const vec1, vec2: Vector3D );
begin
   vecOut.x := vec1.x + vec2.x;
   vecOut.y := vec1.y + vec2.y;
   vecOut.z := vec1.z + vec2.z;
end;
procedure       VectorSubtract( var vecOut:Vector3D;const vec1, vec2: Vector3D );
begin
   vecOut.x := vec1.x - vec2.x;
   vecOut.y := vec1.y - vec2.y;
   vecOut.z := vec1.z - vec2.z;
end;
function        VectorLengthSq( var vec:Vector3D ):real;
begin
   VectorLengthSq := vec.x * vec.x + vec.y * vec.y + vec.z * vec.z;
end;
function        VectorLength( var vec:Vector3D ):real;
begin
   VectorLength := sqrt(vec.x * vec.x + vec.y * vec.y + vec.z * vec.z);
end;
procedure       VectorNormalize( var vec:Vector3D );
var
   flLength: real;
begin
   flLength := VectorLength( vec );

   if ( flLength = 0.0 ) then begin
       vec.x := 1.0;
       vec.y := 0.0;
       vec.z := 0.0;
   end
   else
   begin
       flLength := 1.0 / flLength;
       vec.x := vec.x * flLength;
       vec.y := vec.y * flLength;
       vec.z := vec.z * flLength;
   end;

end;
procedure       VectorScale( var vec:Vector3D; fl:real );
begin
  vec.x := vec.x*fl;
  vec.y := vec.y*fl;
  vec.z := vec.z*fl;
end;
function        VectorDot( const V1,V2:Vector3D ):real;
begin
     VectorDot := (V1.x) * (V2.x) + (V1.y) * (V2.y) + (V1.z) * (V2.z);
end;
procedure       VectorCross( var vec:Vector3D; const V1,V2 : Vector3D );
begin
	vec.x := (V1.y) * (V2.z) - (V1.z) * (V2.y);
	vec.y := (V1.z) * (V2.x) - (V1.x) * (V2.z);
	vec.z := (V1.x) * (V2.y) - (V1.y) * (V2.x);
end;

{ MATRIX }
procedure       MatrixIdentity( var mat:Matrix );
begin
   mat.m11 := 1.0;
   mat.m12 := 0.0;
   mat.m13 := 0.0;
   mat.m14 := 0.0;
   mat.m21 := 0.0;
   mat.m22 := 1.0;
   mat.m23 := 0.0;
   mat.m24 := 0.0;
   mat.m31 := 0.0;
   mat.m32 := 0.0;
   mat.m33 := 1.0;
   mat.m34 := 0.0;
   mat.m41 := 0.0;
   mat.m42 := 0.0;
   mat.m43 := 0.0;
   mat.m44 := 1.0;
end;
procedure       MatrixTranslation( var mat:Matrix; x,y,z:real );
begin
   mat.m11 := 1.0;
   mat.m12 := 0.0;
   mat.m13 := 0.0;
   mat.m14 := 0.0;
   mat.m21 := 0.0;
   mat.m22 := 1.0;
   mat.m23 := 0.0;
   mat.m24 := 0.0;
   mat.m31 := 0.0;
   mat.m32 := 0.0;
   mat.m33 := 1.0;
   mat.m34 := 0.0;
   mat.m41 := x;
   mat.m42 := y;
   mat.m43 := z;
   mat.m44 := 1.0;
end;
procedure       MatrixScaling( var mat:Matrix; sx,sy,sz:real );
begin
   mat.m11 := sx;
   mat.m12 := 0.0;
   mat.m13 := 0.0;
   mat.m14 := 0.0;
   mat.m21 := 0.0;
   mat.m22 := sy;
   mat.m23 := 0.0;
   mat.m24 := 0.0;
   mat.m31 := 0.0;
   mat.m32 := 0.0;
   mat.m33 := sz;
   mat.m34 := 0.0;
   mat.m41 := 0.0;
   mat.m42 := 0.0;
   mat.m43 := 0.0;
   mat.m44 := 1.0;
end;
procedure       MatrixRotationX( var mat:Matrix; angle:real );
var
   fsin,fcos:real;
begin
   fsin := sin( angle );
   fcos := cos( angle );

   mat.m11 := 1.0;
   mat.m12 := 0.0;
   mat.m13 := 0.0;
   mat.m14 := 0.0;
   mat.m21 := 0.0;
   mat.m22 := fcos;
   mat.m23 := fsin;
   mat.m24 := 0.0;
   mat.m31 := 0.0;
   mat.m32 := - fsin;
   mat.m33 := fcos;
   mat.m34 := 0.0;
   mat.m41 := 0.0;
   mat.m42 := 0.0;
   mat.m43 := 0.0;
   mat.m44 := 1.0;
end;
procedure       MatrixRotationY( var mat:Matrix; angle:real );
var
   fsin,fcos:real;
begin
   fsin := sin( angle );
   fcos := cos( angle );

   mat.m11 := fcos;
   mat.m12 := 0.0;
   mat.m13 := -fsin;
   mat.m14 := 0.0;
   mat.m21 := 0.0;
   mat.m22 := 1.0;
   mat.m23 := 0.0;
   mat.m24 := 0.0;
   mat.m31 := fsin;
   mat.m32 := 0.0;
   mat.m33 := fcos;
   mat.m34 := 0.0;
   mat.m41 := 0.0;
   mat.m42 := 0.0;
   mat.m43 := 0.0;
   mat.m44 := 1.0;
end;
procedure       MatrixRotationZ( var mat:Matrix; angle:real );
var
   fsin,fcos:real;
begin
   fsin := sin( angle );
   fcos := cos( angle );

   mat.m11 := fcos;
   mat.m12 := fsin;
   mat.m13 := 0.0;
   mat.m14 := 0.0;
   mat.m21 := -fsin;
   mat.m22 := fcos;
   mat.m23 := 0.0;
   mat.m24 := 0.0;
   mat.m31 := 0.0;
   mat.m32 := 0.0;
   mat.m33 := 1.0;
   mat.m34 := 0.0;
   mat.m41 := 0.0;
   mat.m42 := 0.0;
   mat.m43 := 0.0;
   mat.m44 := 1.0;
end;
procedure       MatrixMultiply( var mat:Matrix;const m1,m2:Matrix );
begin
    mat.m11 :=	m1.m11*m2.m11 + m1.m12*m2.m21 + m1.m13*m2.m31 + m1.m14*m2.m41;
    mat.m12 :=	m1.m11*m2.m12 + m1.m12*m2.m22 + m1.m13*m2.m32 + m1.m14*m2.m42;
    mat.m13 :=	m1.m11*m2.m13 + m1.m12*m2.m23 + m1.m13*m2.m33 + m1.m14*m2.m43;
    mat.m14 :=	m1.m11*m2.m14 + m1.m12*m2.m24 + m1.m13*m2.m34 + m1.m14*m2.m44;

    mat.m21 :=	m1.m21*m2.m11 + m1.m22*m2.m21 + m1.m23*m2.m31 + m1.m24*m2.m41;
    mat.m22 :=	m1.m21*m2.m12 + m1.m22*m2.m22 + m1.m23*m2.m32 + m1.m24*m2.m42;
    mat.m23 :=	m1.m21*m2.m13 + m1.m22*m2.m23 + m1.m23*m2.m33 + m1.m24*m2.m43;
    mat.m24 :=	m1.m21*m2.m14 + m1.m22*m2.m24 + m1.m23*m2.m34 + m1.m24*m2.m44;

    mat.m31 :=	m1.m31*m2.m11 + m1.m32*m2.m21 + m1.m33*m2.m31 + m1.m34*m2.m41;
    mat.m32 :=	m1.m31*m2.m12 + m1.m32*m2.m22 + m1.m33*m2.m32 + m1.m34*m2.m42;
    mat.m33 :=	m1.m31*m2.m13 + m1.m32*m2.m23 + m1.m33*m2.m33 + m1.m34*m2.m43;
    mat.m34 :=	m1.m31*m2.m14 + m1.m32*m2.m24 + m1.m33*m2.m34 + m1.m34*m2.m44;

    mat.m41 :=	m1.m41*m2.m11 + m1.m42*m2.m21 + m1.m43*m2.m31 + m1.m44*m2.m41;
    mat.m42 :=	m1.m41*m2.m12 + m1.m42*m2.m22 + m1.m43*m2.m32 + m1.m44*m2.m42;
    mat.m43 :=	m1.m41*m2.m13 + m1.m42*m2.m23 + m1.m43*m2.m33 + m1.m44*m2.m43;
    mat.m44 :=	m1.m41*m2.m14 + m1.m42*m2.m24 + m1.m43*m2.m34 + m1.m44*m2.m44;
end;
procedure       MatrixLookAtLH( var mat:Matrix; const eye,at,up:Vector3D );
var
   xAxis,yAxis,zAxis:Vector3D;
begin
        VectorSubtract( zAxis, At, Eye );
	VectorNormalize( zAxis );      {	// zaxis = normal(At - Eye)}

        VectorCross( xAxis, Up, zAxis );
	VectorNormalize( xAxis );   {	// xaxis = normal(cross(Up, zaxis))}
	VectorCross( yAxis, zAxis, xAxis );				    {	// yaxis = cross(zaxis, xaxis)}

	mat.m11 := xAxis.x;	mat.m12 := yAxis.x;	mat.m13 := zAxis.x;
	mat.m21 := xAxis.y;	mat.m22 := yAxis.y;	mat.m23 := zAxis.y;
	mat.m31 := xAxis.z;	mat.m32 := yAxis.z;	mat.m33 := zAxis.z;
	mat.m41 := - VectorDot( xAxis, Eye );
	mat.m42 := - VectorDot( yAxis, Eye );
	mat.m43 := - VectorDot( zAxis, Eye );

	mat.m14 := 0.0;
        mat.m24 := 0.0;
        mat.m34 := 0.0;
	mat.m44 := 1.0;
end;
procedure       MatrixPerspectiveFovLH( var mat:Matrix; const fovY,Aspect,zn,zf:real );
var
   w,h:real;
begin
	{*
	w       0       0               0
	0       h       0               0
	0       0       zf/(zf-zn)      1
	0       0       -zn*zf/(zf-zn)  0
	*}

	h := 1.0 / ( sin( fovY / 2.0 ) / cos( fovY / 2.0 ) );
	w := h / Aspect;

	mat.m11 := w;
	mat.m22 := h;
	mat.m33 := zf/(zf-zn);
	mat.m43 := -zn*zf/(zf-zn);
	mat.m34 := 1.0;

	mat.m12 := 0.0;
        mat.m13 := 0.0;
        mat.m14 := 0.0;
	mat.m21 := 0.0;
        mat.m23 := 0.0;
        mat.m24 := 0.0;
	mat.m31 := 0.0;
        mat.m32 := 0.0;
	mat.m41 := 0.0;
        mat.m42 := 0.0;
        mat.m44 := 0.0;
end;

{ Snake Segments }
{delete list of snake segments}
procedure       DeleteSegments( var pList: PSnakeSegment );
var
 pTmp:PSnakeSegment;
begin

 while (pList<>NIL) do begin
   pTmp := pList^.pNext;
   dispose(pList);
   pList := pTmp;
 end;

end;
{find collision}
function        SegmentIntersect( const snake:PSnakeSegment; const a0,a1:Vector3D ):boolean;
var
  v, r0, r1 : Vector3D;
  x         : real;
const
  fPM = 0.01;
begin
  SegmentIntersect := false;

  {bound box}
  if ( MinF( snake^.vecPos.x, snake^.vecFrom.x ) > MaxF(a0.x,a1.x)+fPM ) then exit;
  if ( MaxF( snake^.vecPos.x, snake^.vecFrom.x ) < MinF(a0.x,a1.x)-fPM ) then exit;
  if ( MinF( snake^.vecPos.z, snake^.vecFrom.z ) > MaxF(a0.z,a1.z)+fPM ) then exit;
  if ( MaxF( snake^.vecPos.z, snake^.vecFrom.z ) < MinF(a0.z,a1.z)-fPM ) then exit;

  {rotate scene}
  VectorSubtract( v, a0, snake^.vecFrom );
  r0.x := v.x * snake^.iCos - v.z * snake^.iSin;
  r0.y := v.y;
  r0.z := v.z * snake^.iCos + v.x * snake^.iSin;

  VectorSubtract( v, a1, snake^.vecFrom );
  r1.x := v.x * snake^.iCos - v.z * snake^.iSin;
  r1.y := v.y;
  r1.z := v.z * snake^.iCos + v.x * snake^.iSin;

  {rotated bound box}
  if ( MinF(r0.x,r1.x) > snake^.length+fPM ) then exit;
  if ( MaxF(r0.x,r1.x) < 0.0-fPM ) then exit;
  if ( MinF(r0.z,r1.z) > 0.0+fPM ) then exit;
  if ( MaxF(r0.z,r1.z) < 0.0-fPM ) then exit;

  {find intersection}
  if ( r0.z = r1.z ) then begin
    SegmentIntersect := true;
    exit;
  end;

  x := r0.x - r0.z*(r1.x-r0.x)/(r1.z-r0.z);


  if ( ( x >= 0.0 ) AND ( x < snake^.length ) ) then
    SegmentIntersect := true;

end;

{ DEVICE }
procedure       Device.InitDevice;
begin
  TextMode(CO80+font8x8);

  MatrixIdentity(mat);
  MatrixIdentity(matWorld);
  MatrixIdentity(matView);
  MatrixIdentity(matProj);
end;
procedure       Device.FreeDevice;
begin

end;
procedure       Device.Present;
var
   i:longint;
function Depth2Char( z:word ):byte;
begin
   Depth2Char := 32;
end;
begin
   {copy backbuffer to the screen}
   for i:= 0 to ScrCX*ScrCY-1 do
   begin
     mem[$B800:i*2+1] := bbuffer[i];
     mem[$B800:i*2+0] := ord(tbuffer[i]);
   end;
end;
procedure       Device.SetTransform( ts:integer; const m:Matrix );
var
  im:Matrix;
begin
  case ts of
  TS_PROJECTION: matProj := m;
  TS_VIEW:       matView := m;
  TS_WORLD:      matWorld:= m;
  end;

  MatrixMultiply(im,matView,matProj);
  MatrixMultiply(mat,matWorld,im);
end;
procedure       Device.SetBrushColor( b:byte );
begin
  color := 15+(b shl 4);
end;
procedure       Device.VectorProject( var vecOut:Vector3D; const vec:Vector3D );
begin

  vecOut.x := mat.M11 * vec.x + mat.M21 * vec.y + mat.M31 * vec.z + mat.M41;
  vecOut.y := mat.M12 * vec.x + mat.M22 * vec.y + mat.M32 * vec.z + mat.M42;
  {vecOut.w :}
  vecOut.z := mat.M14 * vec.x + mat.M24 * vec.y + mat.M34 * vec.z + mat.M44;

  if (vecOut.z>1.0) then begin
    vecOut.x := (vecOut.x / vecOut.z + 0.5)*ScrCX;
    vecOut.y := (- vecOut.y / vecOut.z + 0.5)*ScrCY;
  end
  else
  begin
    vecOut.x := (vecOut.x + 0.5)*ScrCX;
    vecOut.y := (- vecOut.y + 0.5)*ScrCY;
  end;
end;
procedure       Device.RenderLine( const v0,v1:Vector3D );
var
  t0,t1:Vector3D;{projected vectors}

  x0,x1,
  y0,y1,
  z0,z1:longint;

  dx, dz : longint;

  i, x, xTo, y, z:longint;
  ix0,ix1:longint;
begin

  {project vectors }
  VectorProject( t0, v0 );
  if ( t0.z < 1.0 ) then exit;
  VectorProject( t1, v1 );
  if ( t1.z < 1.0 ) then exit;

  x0 := round(t0.x);
  y0 := round(t0.y);
  z0 := round(t0.z);
  x1 := round(t1.x);
  y1 := round(t1.y);
  z1 := round(t1.z);


  {clipping 0}
  if ( x0 < 0 ) AND ( x1 < 0 ) then exit;
  if ( y0 < 0 ) AND ( y1 < 0 ) then exit;
  if ( x0 >= (ScrCX) ) AND ( x1 >= (ScrCX) ) then Exit;
  if ( y0 >= (ScrCY) ) AND ( y1 >= (ScrCY) ) then Exit;
  {deltas}
  if ( y0 = y1 ) then begin
    if (x0 = x1) then begin{DRAW POINT}
       i := (x0) + (y0)*ScrCX;
       z0 := Min(z0,z1);
       if ( zbuffer[i] > z0 ) then begin
         bbuffer[i] := color;
         zbuffer[i] := z0;
       end;
       exit;
    end;
    {DRAW HORIZONTAL LINE}
    if (x0 > x1) then
    begin{swap pt0 & pt1}
      i:=x0; x0:=x1; x1:=i;
      i:=y0; y0:=y1; y1:=i;
      i:=z0; z0:=z1; z1:=i;
    end;
    z0 := z0 shl BitShift;
    z1 := z1 shl BitShift;
    dz := (z1 - z0) div (x1 - x0);

    if (x0 < 0) then begin
      z0 := z0 - dz*x0;
      x0 := 0;
    end;
    if (x1 > ScrCX) then x1 := ScrCX;

    i := y0*ScrCX + x0;

    for x := x0 to x1-1 do begin

     z := z0 shr BitShift;
     if (zbuffer[i] > z) then begin
       bbuffer[i] := color;
       zbuffer[i] := z;
     end;

     z0 := z0 + dz;
     inc(i);
    end;

    exit;
  end;

  if (y0 > y1) then
  begin{swap pt0 & pt1}
    i:=x0; x0:=x1; x1:=i;
    i:=y0; y0:=y1; y1:=i;
    i:=z0; z0:=z1; z1:=i;
  end;

  x0 := x0 shl BitShift;
  x1 := x1 shl BitShift;

  z0 := z0 shl BitShift;
  z1 := z1 shl BitShift;

  dz := (z1 - z0) div (y1 - y0);
  dx := (x1 - x0) div (y1 - y0);


  {clipping 1}
  if ( y0 < 0 ) then begin
    x0 := x0 - y0*dx;
    z0 := z0 - y0*dz;
    y0 := 0;
  end;
  if ( y1 > ScrCY ) then
    y1 := ScrCY;

  {rasterize}

  for y := y0 to y1-1 do begin
   xTo := x0+dx;

   if (dx>=0) then begin
     ix0 := x0 shr BitShift;
     ix1 := xTo shr BitShift
   end
   else begin
     ix1 := x0 shr BitShift;
     ix0 := xTo shr BitShift
   end;

   if (ix1>=0) AND (ix0<ScrCX) then begin
     if (ix0<0) then ix0:=0;
     if (ix1>=ScrCX) then ix1:=ScrCX-1;

     i:= ix0 + y*ScrCX;
     z := z0 shr BitShift;

     while (ix0<=ix1) do begin


       if ( zbuffer[i] > z ) then
       begin
         zbuffer[i] := z;
         bbuffer[i] := color;
       end;

       inc(ix0);
       inc(i);
     end;


   end;

   x0 := xTo;
   z0 := z0+dz;
  end;


end;
procedure       Device.DrawText(const x,y:longint; const str:string);
var
  i,ix:longint;
begin
  if (x>=ScrCX) OR (y>=ScrCY) OR (y<0) then Exit;

  ix := x;
  for i := 1 to ord(str[0]) do begin
   if (ix >= 0) AND (ix < ScrCX) then
     tbuffer[y*ScrCX+ix] := str[i];

   inc(ix);
  end;
end;
procedure       Device.Clear;
var
 i:longint;
begin
 for i:= 0 to ScrCX*ScrCY-1 do
 begin
   bbuffer[i] := color;
   zbuffer[i] := $ff;
   tbuffer[i] := ' ';
 end;
end;
{HOUSE}
{delete house list}
procedure DeleteHouses( var houses:PHouse );
var
 h:PHouse;
begin
 while (houses <> NIL) do begin
   h := houses^.pNext;

   dispose(houses);

   houses := h;
 end;
end;
{render the house}
procedure House.Render;
var
  v0,v1 : Vector3D;
begin
  iDevice.SetBrushColor(color);

  v0 := vMin;
  v1 := vMax;

  v1.x := vMin.x;

  iDevice.RenderLine(v0,v1);

  v1.x := vMax.x;
  v1.z := vMin.z;
  iDevice.RenderLine(v0,v1);

  v0.x := vMax.x;
  v0.z := vMax.z;

  v1.x := vMin.x;
  v1.z := vMax.z;
  iDevice.RenderLine(v0,v1);

  v1.x := vMax.x;
  v1.z := vMin.z;
  iDevice.RenderLine(v0,v1);

end;
{find collision house vs. point}
function  House.PtIsIn(const v:Vector3D):boolean;
begin
  PtIsIn := (v.x > vMin.x) AND (v.x < vMax.x) AND (v.z > vMin.z) AND (v.z < vMax.z);
end;

{GAME}
{initialize new game}
procedure Game.NewGame;
var
 i:integer;
 h:PHouse;
begin
     iTurn := 0.0;
     iScore:= 0;
     bAlive:= true;
     bExit := false;

     iSin := 0.0;
     iCos := 1.0;

     speed := 3.0;

     length := 5.0;

     {create houses}
     i := 20;
     while (i>0) do begin

      dec(i);

      new(h);

      h^.vMin.y := 0.0;
      h^.vMin.x := Random(WorldCX)+(Random(100)/100.0);
      h^.vMin.z := Random(WorldCY)+(Random(100)/100.0);
      h^.vMax.y := 0.0;
      h^.vMax.x := h^.vMin.x + 1.0;
      h^.vMax.z := h^.vMin.z + 1.0;

      h^.color := Random(13)+1;

      h^.pNext := houses;
      houses := h;

     end;

     ResetApple;
end;
{delete game objects}
procedure Game.FreeGame;
begin
  DeleteSegments(iSnake);
  DeleteHouses(houses);
end;
{find new place for the apple}
procedure Game.ResetApple;
var
 h:PHouse;
 b:boolean;
begin
 h := houses;
 b := true;

 while (b=true) do begin

   b := false;

   {new position}
   apple.y := 0.0;
   apple.x := Random(WorldCX);
   apple.z := Random(WorldCY);

   {find collision with the houses}
   while (h<>NIL) do begin
     if (h^.PtIsIn(apple)) then
     begin
       b := true;
       break;{apple is in the house -> find new position}
     end;
     h:=h^.pNext;
   end;

 end;

end;
{key event}
procedure Game.OnKey( iKey:char );
begin
  case iKey of
  'a': iTurn:= iTurn + 0.1;{turn left}
  'd': iTurn:= iTurn - 0.1;{turn right}
  'x': bExit:= true;{quit}
  'n': begin;FreeGame;NewGame;end;{new game}
  end;

  {normalize direction angle}
  if (iTurn<0.0) then iTurn := iTurn + K2Pi
  else if (iTurn>=K2Pi) then iTurn := iTurn - K2Pi;

  {recompute sincos of the current direction}
  iSin := sin(iTurn);
  iCos := cos(iTurn);

end;
{render the scene}
procedure Game.Draw;
var
 matView,
 matWorld :Matrix;
 vecEye,
 vecUp,
 vecAt    :Vector3D;
 pSnake   :PSnakeSegment;
 h        :PHouse;
 s        :string;
begin
 {clear backbuffer}
 iDevice.SetBrushColor( 1 );
 iDevice.Clear;

 {set camera}
 if (iSnake<>NIL) then begin

  vecAt := iSnake^.vecPos;

  vecEye.y := vecAt.y + 2.0;
  vecEye.x := vecAt.x - iCos*3.0;
  vecEye.z := vecAt.z - iSin*3.0;

  vecUp.x := 0.0;
  vecUp.y := 1.0;
  vecUp.z := 0.0;
  MatrixLookAtLH(matView,vecEye,vecAt,vecUp);
  iDevice.SetTransform(TS_VIEW,matView);

 end;

 MatrixIdentity(matWorld);
 iDevice.SetTransform(TS_WORLD,matWorld);

 {render the snake}
 iDevice.SetBrushColor(2);
 pSnake := iSnake;
 while (pSnake<>NIL) do begin
   iDevice.RenderLine(pSnake^.vecPos, pSnake^.vecFrom);
   pSnake := pSnake^.pNext;
 end;

 iDevice.DrawText(0,0,'Snake 3D Text version');

 {houses}
 h := houses;
 while (h<>NIL) do begin

  h^.Render;

  h := h^.pNext;
 end;

 {apple}
 iDevice.SetBrushColor(4);
 iDevice.RenderLine(apple,apple);

 Str(iScore,s);
 s := 'Score: '+s;
 iDevice.DrawText(0,1,s);

 {Str(iS,s);
 iDevice.DrawText(0,5,s);}

 {snake killed - write info}
 if (bAlive = false) then begin

   iDevice.DrawText( 2, 5, 'You are dead...' );
   {Str(integer,s);
   s := 'length: '+s+'m';
   iDevice.DrawText( 2, 6, s );}
   iDevice.DrawText( 2, 8, 'press (n)ew game, e(x)it' );

 end;

 {blt}
 iDevice.Present;
end;
{moves with the game objects}
procedure Game.FrameMove( flFrameLength:real );
var
 pSnake:PSnakeSegment;
 vec   :Vector3D;
 fl    :real;
 h     :PHouse;
begin
 if (bAlive = true) then begin
 {}
 if ( iSnake = NIL ) OR {( iSnake^.turn <> iTurn ) OR} (iSnake^.length > 0.1) then
 begin
     if (iSnake = NIL) then begin
       vec.x := 0.0;
       vec.y := 0.0;
       vec.z := 0.0;
     end
     else
       vec := iSnake^.vecPos;

     {add snake segment}
     new(pSnake);
     pSnake^.vecPos := vec;
     pSnake^.vecFrom := vec;
     pSnake^.length := 0.0;
     pSnake^.turn := iTurn;
     pSnake^.iSin := iSin;
     pSnake^.iCos := iCos;
     pSnake^.pNext := iSnake;

     iSnake := pSnake;
 end;

 fl := speed*flFrameLength;
 iSnake^.vecPos.x := iSnake^.vecPos.x + iCos*fl;
 iSnake^.vecPos.z := iSnake^.vecPos.z + iSin*fl;
 iSnake^.length := iSnake^.length + fl;

 fl := 0.0;
 pSnake := iSnake;
 while (pSnake <> NIL) do begin
   fl := fl + pSnake^.length;

   if (fl > length) then begin
      fl := pSnake^.length - (fl-length);
      pSnake^.length := fl;
      vec.y := 0.0;
      vec.x := - pSnake^.iCos*fl;
      vec.z := - pSnake^.iSin*fl;
      VectorAdd(pSnake^.vecFrom,pSnake^.vecPos,vec);

      DeleteSegments(pSnake^.pNext);

   end;

   pSnake := pSnake^.pNext;
 end;

 {colision detection}
 if (iSnake <> NIL) then begin
   pSnake := iSnake^.pNext;
   {snake body}
   {iS:=1;}

   while (pSnake<>NIL) do begin
     VectorSubtract( vec, iSnake^.vecPos, pSnake^.vecPos );

     if ( VectorLength(vec) > 0.3 ) then break;

     {inc(iS);}
     pSnake := pSnake^.pNext;
   end;

   while (pSnake<>NIL) do begin

     if (SegmentIntersect(pSnake, iSnake^.vecFrom, iSnake^.vecPos)) then
     begin
       bAlive := false;
       pSnake := NIL;
     end
     else
       pSnake := pSnake^.pNext;
       {inc(iS);}
   end;

 end;

 {houses}
   h := houses;
   while (h<>NIL) do begin
    if (h^.PtIsIn(iSnake^.vecPos)) then begin
       bAlive := false;
       h := NIL;
    end
    else
      h:=h^.pNext;
   end;
   {apple}
   VectorSubtract(vec,iSnake^.vecPos,apple);
   if (VectorLength(vec) < 0.2) then begin
    inc(iScore);
    length := length + 2.0;
    ResetApple;
   end;
 {}
 end
 else begin

 end;

end;
{setup screen}
procedure       InitGraphics;
var
 mat:Matrix;
begin
 iDevice.InitDevice;{init render device}

 {setup projection transform matrix}
 MatrixPerspectiveFovLH( mat, 3.1415/4.0, 4.0/3.0, 1.0, 100.0 );
 iDevice.SetTransform(TS_PROJECTION,mat);

end;
{main(void)}
begin

  Randomize;

  InitGraphics; {setup screen}
  iGame.NewGame;{start new game}

  while ( iGame.bExit = false ) do begin  {user wouldnt exit}

    if KeyPressed then iGame.OnKey(ReadKey);{translate key}

    iGame.FrameMove(0.005);{scene move}
    iGame.Draw;{render scene}
    Delay(20);{fps delay}
  end;

  iGame.FreeGame;{kill the game}

end.